<?php
/**
 * @version        mod_JExtCurrency.php 3 may 2009
 * @package        JExt
 * @copyright      Copyright (C) 2009-2010 JExt. All rights reserved.
 * @license        GNU/GPL, see LICENSE.php
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');


/**
 * @example
 * $w = new Currency();
 * $w->enable_cache = 1;
 * $w->cache_path = '/var/www/mysite.com/cache';
 * $ar_data = $w->get_currency_data();
 * print_r($ar_data);
 *
 */

class Weather {

    /**
     * Включение кэширования
     *
     * @var boolean
     */
    var $enable_cache = 1;

    /**
     * Путь до директории кэширования
     * eg. /www/website.com/cache
     *
     * @var string
     */
    var $cache_path = '';

    /**
     * Время кэширования в секундах
     * Default: 3600 = 1 час
     *
     * @var int
     */
    var $cache_time = 3600; // 1 hour

    /**
     * Полный путь к кэшированному файлу
     *
     * @var string
     */
    var $cache_file;

    /**
     * URL для получения xml данных
     *
     * @var string
     */
    var $weather_api_url = "";

    /**
     * Переменная для хранения данных от curl запроса с сервера
     *
     * @var string
     */
    var $raw_data;

    var $return_array = array();
    
    var $currency_class;
    /**
     * Получение информации о курсах
     *
     * если enable_cache = true, данные кэшируются и обновляются
     * в зависимости от установленного времени кэширования
     *
     * Данные возвращаются в ассоциативном массиве
     *
     * @return array
     */
    
	/**
 	* Id города
 	* @var int
 	*/
    var $city_id;
    
    /**
     * Колличество дней
     * @var int
     */
    var $dayf;
    
    /**
     * Название вашего сайта
     * @var string
     */
    var $site_id;
    
    /**
     * язык экспорта
     * @var string
     */
    var $lang;
    
    function get_weather_data($source_class) {

        //подключаем используемый класс
        require_once (JPATH_SITE.DS.'modules'.DS.'mod_jextweather'.DS.'plugin'.DS.'main.class.php');
        require_once (JPATH_SITE.DS.'modules'.DS.'mod_jextweather'.DS.'plugin'.DS.$source_class.'.class.php');

        $filename = "jextweather_".$this->filename_safe($source_class)."_".$this->city_id."_".$this->dayf."_".$this->lang;


        if ($this->enable_cache && !empty($this->cache_path)) {

            $this->cache_file = $this->cache_path . DS . $this->filename_safe($filename);

            if (JFile::exists($this->cache_file)) {

                $cached_data = $this->load_from_cache();

                if($cached_data) return $cached_data;

            }
        }

        $classname = "weather_data_".$source_class;
        $this->weather_class = new $classname ();

        $this->weather_api_url = $this->weather_class->getUrl($this->city_id,$this->dayf,$this->site_id,$this->lang);
        //Получаем данные о курсах действующие
        $this->get_data();
        //print_r($this->return_array);

        if (!isset($this->return_array['error']) && $this->enable_cache && !empty($this->cache_path)) {
            $this->write_to_cache($this->return_array);
        }

        return $this->return_array;


    }

    /**
     * Получение данных о курсах валют с сервера
     * @return array
     */
    function get_data() {

        if(function_exists('curl_init')) {
            if ($this->make_request()) {

                $xml = new JSimpleXML();

                $xml->loadString($this->raw_data);

                $this->return_array = $this->weather_class->get_array($xml);

                if(isset ($this->return_array['error'])) {
                    $this->return_array['error']=$this->return_array['error'];
                }
            }
            else {
                $this->return_array['error']="CONNECT_ERROR";
            }

        }else $this->return_array['error']="CURL_ERROR";

    }

    function load_from_cache() {

        $file_time = filectime($this->cache_file);
        $now = time();
        $diff = ($now-$file_time);

        if ($diff <= $this->cache_time) {
            return unserialize(JFile::read($this->cache_file));
        } else {
            JFile::delete($this->cache_file);
            return false;
        }
    }

    function write_to_cache($return_array) {

        if (!JFile::exists($this->cache_path)) {

            JFolder::create($this->cache_path, 0777);
        }

        if (!JFile::write($this->cache_file, serialize($return_array))) {
            $this->return_array['error']= "CACHE_ERROR";
        }
    }

    function make_request() {

            $curl = curl_init();
            // saves us before putting directly results of request
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
            // url to get
            curl_setopt($curl, CURLOPT_URL, $this->weather_api_url);
            // timeout in seconds
            curl_setopt($curl, CURLOPT_TIMEOUT, 20);
            // useragent
            curl_setopt($curl, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
            // reading content
            $this->raw_data = curl_exec($curl);
            // closing connection
            curl_close($curl);

        $this->incode();

        if (empty($this->raw_data)) {
            return false;
        }else {
            return true;
        }

    }

    function incode() {

        if(stripos($this->raw_data,"windows-1251")) {
            $this->raw_data = iconv("windows-1251","utf-8",$this->raw_data);
            $this->raw_data = str_replace("windows-1251","utf-8",$this->raw_data);
        }
        elseif(stripos($this->raw_data,"windows-1252")) {
            $this->raw_data= iconv("windows-1252","utf-8",$this->raw_data);
            $this->raw_data = str_replace("windows-1252","utf-8",$this->raw_data);
        }
        elseif(stripos($this->raw_data,"koi8-u")) {
            $this->raw_data= iconv("koi8-u","utf-8",$this->raw_data);
            $this->raw_data = str_replace("koi8-u","utf-8",$this->raw_data);
        }

    }

    function filename_safe($filename) {
        $temp = $filename;
        $temp = strtolower($temp);
        $temp = str_replace(" ", "_", $temp);
        $result = '';
        for ($i=0; $i<strlen($temp); $i++) {
            if (preg_match('([0-9]|[a-z]|_)', $temp[$i])) {
                $result = $result . $temp[$i];
            }
        }
        return $result;
    }
    
}
?>