<?php
/*------------------------------------------------------------------------
# mod_sp_weather - Weather Module by JoomShaper.com
# ------------------------------------------------------------------------
# author    JoomShaper http://www.joomshaper.com
# copyright Copyright (C) 2010 JoomShaper.com. All Rights Reserved.
# @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
# Websites: http://www.joomshaper.com - http://www.joomxpert.com
-------------------------------------------------------------------------*/

// no direct access
defined('_JEXEC') or die('Restricted access');

class modSPWeatherHelper
{	
	var $config;
	var $location;
	var $data = array();
	
	//Initiate configurations
	function init(&$params) {
		jimport('joomla.filesystem.file');
		$this->config = array(
            'uniqid'	=> $params->get('uniqid',1),
            'location'	=> str_replace(' ','%20',$params->get('location','London,UK')),
            'language' 	=> $params->get('language','en'),
            'tempUnit' 	=> $params->get('tempUnit','c'),
			'useCache' => $params->get('useCache',1),
			'cacheTime' => $params->get('cacheTime',5)			
		);
	}

	//Curl function
	function curl_init() {
		if(function_exists('curl_init'))
		{
			// initializing connection
			$curl = curl_init();
			// saves us before putting directly results of request
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
			// url to get
			curl_setopt($curl, CURLOPT_URL, 'http://www.google.com/ig/api?weather=' . $this->config['location'].'&hl='.$this->config['language']);
			// timeout in seconds
			curl_setopt($curl, CURLOPT_TIMEOUT, 20);
			// useragent
			curl_setopt($curl, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
			// reading location
			$this->location = curl_exec($curl);
			// closing connection
			curl_close($curl);
		}
		else
		{
			$this->data['error'] = 'cURL extension is not available on your server';
		}
	}
	
	//Get Weather datas
	function loadData()
	{
		//if used cache
		if($this->config['useCache'] == 1)
		{
			//path to save cache file
			$cache_path='modules/mod_sp_weather/cache/mod_sp_weather' . $this->config['uniqid'] . '.xml';
			//checking for file exists
			if (!file_exists($cache_path)){
				modSPWeatherHelper::curl_init();
				JFile::write($cache_path, $this->location);
			} elseif (filesize($cache_path) == 0 || ((filemtime($cache_path) + $this->config['cacheTime'] * 60) < time())){
				modSPWeatherHelper::curl_init();
				JFile::write($cache_path, $this->location);
			}
			//load xml
			$this->location = JFile::read($cache_path);
		}
		else
		{
			modSPWeatherHelper::curl_init();
		}
	}	
	
	
	function get_weather_data() {
		
		$xml =& JFactory::getXMLParser('Simple');
		
		if($xml->loadString($this->location))
		{
			// checking data correct
			if(!isset($xml->document->weather[0]->problem_cause[0]))
			{				
				// checking data correct
				$forecast_information = $xml->document->weather[0]->forecast_information[0];
				$current_conditions = $xml->document->weather[0]->current_conditions[0];
				$forecast_conditions = $xml->document->weather[0]->forecast_conditions;
					 
				if(
					isset($forecast_information->unit_system[0]) &&
					isset($current_conditions->condition[0]) &&
					isset($current_conditions->temp_f[0]) &&
					isset($current_conditions->temp_c[0]) &&
					isset($current_conditions->humidity[0]) &&
					isset($current_conditions->icon[0])
				)
				{							 
					 
					$this->data['f_info']['city'] = $forecast_information->city[0]->attributes('data');
					$this->data['f_info']['unit'] = $forecast_information->unit_system[0]->attributes('data');

					$this->data['c_conditions']['condition'] = $current_conditions->condition[0]->attributes('data');
					$this->data['c_conditions']['temp_f'] = $current_conditions->temp_f[0]->attributes('data');
					$this->data['c_conditions']['temp_c'] = $current_conditions->temp_c[0]->attributes('data');
					$this->data['c_conditions']['humidity'] = $current_conditions->humidity[0]->attributes('data');
					$this->data['c_conditions']['icon'] = modSPWeatherHelper::icon($current_conditions->icon[0]->attributes('data'),'modules/mod_sp_weather/assets/images/');
					//Absense of this feature
					if (isset($current_conditions->wind_condition[0])) {
						$this->data['c_conditions']['wind'] = $current_conditions->wind_condition[0]->attributes('data');								
					}
					for ($i = 0; $i < count($forecast_conditions); $i++){
						$data = $forecast_conditions[$i];
						$this->data['forecast'][$i]['day_of_week'] = $data->day_of_week[0]->attributes('data');
						$this->data['forecast'][$i]['low'] = $data->low[0]->attributes('data');
						$this->data['forecast'][$i]['high'] = $data->high[0]->attributes('data');
						$this->data['forecast'][$i]['icon'] = modSPWeatherHelper::icon($data->icon[0]->attributes('data'),'modules/mod_sp_weather/assets/images/icons/');
						$this->data['forecast'][$i]['condition'] = $data->condition[0]->attributes('data');
					}
						
				} else {
					$this->data['error'] = 'An error occured during parsing XML data. Please try again.';
				}
				
			}
			else 
			{
				$this->data['error'] = 'An error occured - you set wrong location or data for your location are unavailable';
			}
			
		} else {
			$this->data['error'] = 'Parse error in downloaded data';
		}				
			
		
		return $this->data;	
	}
	
	//Get weather
	function getWeather() {
	    $weather_data = modSPWeatherHelper::get_weather_data(); 
        $weather = new stdClass();
		//Checking for errors
        if (isset($weather_data['error'])) { 
       	     $weather->error = $weather_data['error'];
       	 } else {
    	     $weather->city = $weather_data['f_info']['city'];
    	     $weather->unit = $weather_data['f_info']['unit'];
    	     
       	     $weather->c_temp_f = $weather_data['c_conditions']['temp_f'];
       	     $weather->c_temp_c = $weather_data['c_conditions']['temp_c'];
       	     $weather->c_condition = $weather_data['c_conditions']['condition'];
             $weather->c_humidity = $weather_data['c_conditions']['humidity'];
             $weather->c_icon = $weather_data['c_conditions']['icon'];
             $weather->c_wind = @$weather_data['c_conditions']['wind'];
             //Getting forecasts
             if (is_array($weather_data['forecast'])) {
    	         $weather->forecast = $weather_data['forecast'];
    	     }
   	     }
   	     return $weather;
          
	}	
	
	//Convert temparature
	function convert($value,$unit,$tempType)
	{	
		if($unit=='US' && $tempType == 'c')
		{
			return floor((5/9) * ($value - 32)).'&deg;C';
		}
		else if($unit=='SI' && $tempType == 'f')
		{
			return floor(32 + ((5/9) * $value)).'&deg;F';
		}
		else
		{
			return $value.(($tempType == 'c') ? '&deg;C' : '&deg;F' );
		}		
	}

	//New icon path
	function icon($icon,$path) {
		if (strpos($icon,"/ig/images/weather")!==false) {
			$icon = str_replace("/ig/images/weather","",$icon);
		}
		$icon = str_replace('.gif','.png',$icon);
		return $path . $icon;
	}
}
 